import {
  mergeWindupsToSuffixedWindup,
  PlayedSuffixedWindupMember,
  playedElements,
} from "../SuffixedWindup";
import { windupFromString, fastForward, PlayedWindup, Windup } from "../Windup";
import { suffixedPrevBy, deepPair } from "./util";

type PairElement<E, M> = Windup<E, M> | PlayedWindup<E, M>;
type TestElement<E, M> = [PairElement<E, M>, PairElement<E, M>];
type Expected<E, M> = PlayedSuffixedWindupMember<E, M>[];
type Vector<E, M> = [string, TestElement<E, M>, number, Expected<E, M>];

const finishedSentence = fastForward(
  windupFromString("ABCDEF", {
    m: true,
  })
);
const unfinishedSentence = windupFromString("ABXYEF", {
  m: true,
});

const vectors: Vector<string, { m: boolean }>[] = [
  ["Played", [finishedSentence, unfinishedSentence], 2, ["A", "B"]],
  ['Deep',[fastForward(deepPair.base), deepPair.against], 6, [
    'H', 'i', '!', ' ',
    [['W', 'h', 'a', 't', ' '], [], {m: false}],
  ] ]
];

describe("playedElements", () => {
  vectors.forEach((vector) => {
    const [title, [base, against], prevBy, expected] = vector;
    
    const toRewindEl = mergeWindupsToSuffixedWindup(base, against);
    const toResumeEl = mergeWindupsToSuffixedWindup(against, base)

    it(title, () => {
      const rewound = suffixedPrevBy(toRewindEl, prevBy);

      const rewoundPlayedEls = playedElements(rewound);


      const resumedPlayedEls = playedElements(toResumeEl);
      
 

      expect(rewoundPlayedEls).toEqual(expected);
      expect(resumedPlayedEls).toEqual(expected);
    });
  });
});
